/**
 * Copyright (c) 2016-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.registry.overlay;

import org.ow2.petals.microkernel.api.communication.sharedarea.exception.SharedAreaException;
import org.ow2.petals.microkernel.registry.overlay.api.RegistryOverlayAPI;
import org.ow2.petals.registry_overlay.api.StoredRegistryEndpoint;
import org.ow2.petals.registry_overlay.api.StoredRegistryEndpoint.Type;
import org.w3c.dom.Document;

/**
 * 
 * Internal endpoints
 * 
 */
public class InternalEndpointImpl extends RegistryOverlayEndpoint {

    private static final long serialVersionUID = -6364121452745414838L;

    /**
     * This attribute is defined "transient" to avoid its serialization when sending message exchanges.
     * 
     * It is saved/loaded from the registry by the endpoint service.
     */
    private transient Document description;

    public InternalEndpointImpl(final StoredRegistryEndpoint endpoint) {
        super(EndpointType.INTERNAL, endpoint);
        assert endpoint.getType() == Type.INTERNAL;
    }

    synchronized Document getDescription(final RegistryOverlayAPI sharedAreaOverlay) {
        if (this.description == null) {
            try {
                this.description = sharedAreaOverlay.retrieveEndpointDescription(getServiceName(), getEndpointName());
            } catch (final SharedAreaException e) {
                this.log.warning("Error caught when retrieving the endpoint description of: " + getEndpointName(), e);
            }
        }
        return this.description;
    }
}
