/**
 * Copyright (c) 2016-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.registry.overlay;

import java.util.logging.Logger;

import javax.xml.namespace.QName;

import org.ow2.petals.jbi.servicedesc.endpoint.Location;
import org.ow2.petals.jbi.servicedesc.endpoint.PetalsServiceEndpoint;
import org.ow2.petals.jbi.servicedesc.endpoint.impl.JBIServiceEndpointImpl;
import org.ow2.petals.microkernel.api.jbi.messaging.endpoint.EndpointDirectoryService;
import org.ow2.petals.registry_overlay.api.StoredRegistryEndpoint;

import com.ebmwebsourcing.easycommons.log.LoggingUtil;

/**
 * An implementation of {@link PetalsServiceEndpoint}, to be used with the Petals Registry Overlay, encapsulating its
 * description and loading it lazily.
 */
public abstract class RegistryOverlayEndpoint extends JBIServiceEndpointImpl {

    private static final long serialVersionUID = 2884988166109396382L;

    // deserialization will recreate the object
    protected final transient LoggingUtil log = new LoggingUtil(
            Logger.getLogger(EndpointDirectoryService.COMPONENT_LOGGER_NAME));

    public RegistryOverlayEndpoint(final EndpointType type, final StoredRegistryEndpoint endpoint) {
        super(type, endpoint.getEndpointName(), QName.valueOf(endpoint.getServiceName()),
                toQNames(endpoint.getInterfaces()), new Location(endpoint.getContainer(), endpoint.getComponent()));
    }

    private static QName[] toQNames(final String[] qnames) {
        final int l = qnames.length;
        final QName[] res = new QName[l];
        for (int i = 0; i < l; i++) {
            res[i] = QName.valueOf(qnames[i]);
        }
        return res;
    }
}
